#Production model 

#Calculates average production of benthic algae per hour in the upper 2mm of the sediment 
#Production model: EP
#Adapted from Forster et al. 2006

#Required input:
# - ambient irradiance (hourly averages)
# - emersion yes/no (1/0) per hour of the location over a certain time period. the first hour after emersion should be considered as immersed (as it is assumed that upward migration takes place)
# - chlorofyl-a concentrations in the upper 2 mm of the sediment
# - mud content (fraction < 63 um)
# - photosynthetic parameters alpha, eopt and pmax

#Initialisation --------------------------------------------------------------------------------------------------------------------------------------------------------

require(plyr)

#Set working directory ##################################################################################################################
#Insert your local working directory containing the provided input data files
setwd("//working/directory")

#Load chlorophyll-a concentrations and mud content

chla <- read.csv("//working/directory/slib_chla_03042015.csv")
chla$plotnr <- substr(chla$Customer.Code,1,3)
chla$plotnr <- sapply(chla$plotnr, toupper)

slib <- read.csv("//working/directory/slib_03042015_methooglaag.csv",sep=";")

#Load photosynthetic parameters
EP_pars0304_sort <- read.csv("//working/directory/EP_pars0304_allplots.csv")
Estuary <- c(rep("Eastern Scheldt",18),rep("Western Scheldt",72),rep("Eastern Scheldt",18),rep("Western Scheldt",18),rep("Eastern Scheldt",18),rep("Western Scheldt",18))
Estuary2 <- c(rep("Eastern Scheldt",6),rep("Western Scheldt",24),rep("Eastern Scheldt",6),rep("Western Scheldt",6),rep("Eastern Scheldt",6),rep("Western Scheldt",6))
EP_pars0304_sort$Estuary <- Estuary
slib$Estuary <- Estuary

EP_pars0304_sort_WES <- EP_pars0304_sort[EP_pars0304_sort$Estuary == "Western Scheldt",]

slib_WES <- slib[slib$Estuary == "Western Scheldt",]

#Ambient irradiance
PAR <- read.csv("//working/directory/Lichtintensiteit_meetperiode.csv")
PAR <- PAR[rev(rownames(PAR)),]

Chla <- ddply(chla,~plotnr,summarise,mean=mean(Chla_m2),sd=sd(Chla_m2))

Chla$Estuary <- Estuary2

Chla_WES <- Chla[Chla$Estuary == "Western Scheldt",]
Chla_WES <- rep(Chla_WES$mean,each=3)

mud_WES <- slib_WES$SSILT63_2....

#Get emersion plots

load("//working/directory/emersion_WES_hour_plots_firsthour_excl.RData")

emersion_WES <- emersion_WES[25:744,]


#Put input data in the right format-------------------------------------------------------------------------------------------------------------------------------------------

z2 <- seq(0,2,0.01)#depth in mm

#Get kd based on mud content and average chl-a concentration (based on linear regression Kd data from Eden estuary (Jacco Kromkamp))
Kd_station_WES <- -3.1167 + 1.8298*log(Chla_WES)

#Calculate ps
ps_WES <- as.numeric(as.character(EP_pars0304_sort_WES$ps))
alpha_WES <- as.numeric(as.character(EP_pars0304_sort_WES$alpha))
eopt_WES <- as.numeric(as.character(EP_pars0304_sort_WES$eopt))


#Respiration rate
res_par <- 0.05           #fraction of pmax that determines respiration rate
RES_WES <- res_par*ps_WES# calculate respiration rate by multiplying pmax with 0.05


#---------------------------------------------------------------------------------------------------------------------------------------------------------------------------

##### -----     PP model     ----- ######-----------------------------------------------------------------------------------------------------------------------------------


#Calculate light intensity for each vertical layer: ID = I0 * e^(-kD) 


light_mat <- t(as.matrix(PAR$Light))
light_rep_mat <- light_mat[rep(1, 200),]


#Calculate vertical distribution chlorophyll-a

dmax <- 1.99 #maximum depth in mm
mud_WES <- 0.01 * mud_WES #percentage of < 63 m particles in sample / 100
e <- 2.718281


#CALCULATE RELATIVE CUMULATIVE CHL-A CONCENTRATIONS PER VERTICAL LAYER

mud_mat <- t(as.matrix(mud_WES))
mud_rep_mat <- mud_mat[rep(1, 201),]
z2_mat <- as.matrix(z2)
z2_rep_mat <- z2_mat[,rep(1, length(mud_WES))]
dmax2 <- 2

all_chla <- z2_rep_mat/dmax2 + mud_rep_mat*(1-z2_rep_mat/dmax2-exp((-2*e*z2_rep_mat)/dmax2))

rm(z2_rep_mat,mud_rep_mat,z2_mat,mud_mat)

#CALCULATE RELATIVE CHL-A CONCENTRATION PER LAYER (SO NON-CUMULATIVE)
all_relative_chla <- all_chla[2:nrow(all_chla),] - all_chla[1:(nrow(all_chla)-1),]


## relative to absolute chla distribution
all_absolute_chla<-  data.frame(matrix(ncol = 0, nrow = 200))

Chla_mat <- t(as.matrix(Chla_WES))
Chla_rep_mat <- Chla_mat[rep(1, 200),]
rel_chla_sum <- colSums(all_relative_chla)
rel_chla_sum_mat<- t(as.matrix(rel_chla_sum))
sum_rep_mat <- rel_chla_sum_mat[rep(1, 200),]
Chla_rep <- Chla_mat[rep(1, 200),]

all_absolute_chla_WES <- all_relative_chla*Chla_rep/sum_rep_mat

rm(Chla_mat,Chla_rep_mat,rel_chla_sum,rel_chla_sum_mat,sum_rep_mat,Chla_rep,all_relative_chla,all_chla)


########CALCULATE PRODUCTION USING EP FORMULATION----------------------------------------------------------------------

alpha_WES[is.na(alpha_WES)] <- 0
eopt_WES[is.na(eopt_WES)] <- 0
ps_WES[is.na(ps_WES)] <- 0

RES_WES[is.na(RES_WES)] <- 0


#Calculate production WES

P_pixels_WES <- matrix(nrow=0,ncol=1)

for (kk in 1:length(eopt_WES)){  
  
  Ez <- light_rep_mat*exp(-Kd_station_WES[kk]*z) 
  
  chla <- as.matrix(all_absolute_chla_WES[,kk])
  chla_rep_mat <- chla[,rep(1, 720)]
  
  response <- Ez/((1/(alpha_WES[kk]*eopt_WES[kk]^2))*Ez^2+t((1/ps_WES[kk]-2/(alpha_WES[kk]*eopt_WES[kk]))*t(Ez))+(1/alpha_WES[kk]))*chla_rep_mat
  response_sum <- (colSums(response)-RES_WES[kk]) *0.043
  PP_allpixels_time <- emersion_WES[,kk]*response_sum
  P_pixels_WES <- rbind(P_pixels_WES,sum(PP_allpixels_time))
  print(kk)
}



P_pixels_WES[is.na(P_pixels_WES)] <- 0

slib_WES$plotshort <- substr(slib_WES$Customer.Code,1,3)

P_pixels_WES_plot <- as.data.frame(cbind(P_pixels_WES,slib_WES$plotshort))

P_pixels_WES_plot$V1 <- as.numeric(as.character(P_pixels_WES_plot$V1))
P_pixels_WES_plot$V2 <- as.character(P_pixels_WES_plot$V2)

P_pixels_WES_plot <- P_pixels_WES_plot[!is.na(P_pixels_WES_plot$V1),]

WES_production <- ddply(P_pixels_WES_plot, ~V2,summarise,mean=mean(V1),sd=sd(V1))

WES_production$day <- WES_production$mean/720*24

write.table(WES_production,file="//working/directory/Production_plots_12031004_field_perday_WES.txt")
save(WES_production,file="//working/directory/Production_plots_12031004_field_perday.RData")




