#Production model 

#Calculates average production of benthic algae per hour in the upper 2mm of the sediment 
#Production model: EP
#Adapted from Forster et al. 2006

#Required input:
# - ambient irradiance (hourly averages)
# - emersion yes/no (1/0) per hour of the location over a certain time period. the first hour after emersion should be considered as immersed (as it is assumed that upward migration takes place)
# - chlorofyl-a concentrations in the upper 2 mm of the sediment
# - mud content (fraction < 63 um)
# - photosynthetic parameters alpha and eopt

#Initialisation --------------------------------------------------------------------------------------------------------------------------------------------------------
require(plyr)

#Set working directory ##################################################################################################################
#Insert your local working directory containing the provided input data files
setwd("//working/directory")


#Calculate production WES #######################################################################################################################


#Load reflection band 1,2,3,4 and 5

LS8_band2_WES <- read.csv("//working/directory/l2_WES_noveg.txt")
LS8_band3_WES <- read.csv("//working/directory/l3_WES_noveg.txt")
LS8_band4_WES <- read.csv("//working/directory/l4_WES_noveg.txt")
LS8_band5_WES <- read.csv("//working/directory/l5_WES_noveg.txt") 

#Temperature
temp_knmi <- read.csv("//working/directory/temperature_knmi.csv")#

#Externe lichtintensiteit
PAR <- read.csv("//working/directory/Lichtintensiteit_meetperiode.csv")

#Get emersion per pixel 
load("//working/directory/emersion_WES_hour_firsthour_excl_noveg_correct_bathy.RData")

#remove March 11th (out of study period)
emersion_WES <- emersion_WES[25:744,]


#Put input data in the right format-------------------------------------------------------------------------------------------------------------------------------------------

#Divide reflection values by 10000

LS8_band2_WES$grid_code <- LS8_band2_WES$grid_code/10000
LS8_band3_WES$grid_code <- LS8_band3_WES$grid_code/10000
LS8_band4_WES$grid_code <- LS8_band4_WES$grid_code/10000
LS8_band5_WES$grid_code <- LS8_band5_WES$grid_code/10000

z2 <- seq(0,2,0.01)#depth in mm

alpha <- 0.1705609 #average value of all data

eopt <- 779.2119 #average value of all data

#Get light intensity time series for period of interest

PAR$Time <- as.numeric(PAR$Time)

#Calculate NDVI

NDVI <- (LS8_band5_WES$grid_code - LS8_band4_WES$grid_code)/(LS8_band5_WES$grid_code+ LS8_band4_WES$grid_code)
NDWI <- (LS8_band3_WES$grid_code - LS8_band5_WES$grid_code)/(LS8_band3_WES$grid_code+ LS8_band5_WES$grid_code)


LS8_band2_WES <- LS8_band2_WES[which(NDVI < 3),]
LS8_band3_WES <- LS8_band3_WES[which(NDVI < 3),]
LS8_band4_WES <- LS8_band4_WES[which(NDVI < 3),]
LS8_band5_WES <- LS8_band5_WES[which(NDVI < 3),]


LS8_band2_WES <- LS8_band2_WES[which(NDWI < 0),]
LS8_band3_WES <- LS8_band3_WES[which(NDWI < 0),]
LS8_band4_WES <- LS8_band4_WES[which(NDWI < 0),]
LS8_band5_WES <- LS8_band5_WES[which(NDWI < 0),]

emersion_WES <- emersion_WES[,which(NDVI < 3)]
emersion_WES <- emersion_WES[,which(NDWI < 0)]

NDVI <- NDVI[which(NDVI < 3)]
NDVI <- NDVI[which(NDWI < 0)]



total_emersion_pixel <- colSums(emersion_WES)


#Convert NDVI to chl-a (mg/m2)

Chla <- 556.93374*NDVI + 30.26882

for (i in 1:length(Chla)){
  
  if (Chla[i] < 0){
    Chla[i] <- 0
  }
}
#Calculate mud content

mud  <-  -116    -   488  * LS8_band2_WES$grid_code   +   1488  * LS8_band5_WES$grid_code 

for (i in 1:length(mud)){
  
  if (mud[i] < 0){
    mud[i] <- 0
  }
  if (mud[i] > 100){
    mud[i] <- 100
  }
}

#Get kd based on the average chl-a concentration (based on linear regression Kd data from Eden estuary (Jacco Kromkamp))
Kd_station <- -3.1167 + 1.8298*log(Chla)

PAR$newdate <- strptime(as.character(PAR$Date), "%d/%m/%Y")
PAR$txtdate <- format(PAR$newdate, "%Y%m%d")
ext_PAR <- PAR

#Select temperature from certain time frame

colnames(temp_knmi)[8] <- "temp"
temp_all <- temp_knmi$temp[ temp_knmi$YYYYMMDD %in%  PAR$txtdate]
temp_all <- temp_all/10

#Calculate ps
Topt  <-25 #i2
Tmax  <- 38 #i3
beta  <- 2.75 #i4
pMAX  <- 4.98 #i5

ps <- pMAX *(((Tmax - temp_all)/(Tmax-Topt))^beta) * exp(-beta*(((Tmax - temp_all)/(Tmax-Topt))-1))

#Convert photosynthetic capacity in mol photon m-2 s-1 to photosynthetic capacity (mg C mg-1 chla h-1)
ps <- ps*23.25581

#Respiration rate
res_par <- 0.05           #fraction of pmax that determines respiration rate
RES <- res_par*ps   # calculate respiration rate by multiplying pmax with 0.05



#---------------------------------------------------------------------------------------------------------------------------------------------------------------------------

##### -----     PP model     ----- ######-----------------------------------------------------------------------------------------------------------------------------------


#Calculate light intensity for each vertical layer: ID = I0 * e^(-kD) 


light_mat <- t(as.matrix(ext_PAR$Light))
light_rep_mat <- light_mat[rep(1, 200),]


#Calculate vertical distribution chlorophyll-a

dmax <- 1.99 #maximum depth in mm
mud <- 0.01 * mud #percentage of < 63 m particles in sample / 100!
e <- 2.718281


#CALCULATE RELATIVE CUMULATIVE CHL-A CONCENTRATIONS PER VERTICAL LAYER
mud_mat <- t(as.matrix(mud))
mud_rep_mat <- mud_mat[rep(1, 201),]
z2_mat <- as.matrix(z2)
z2_rep_mat <- z2_mat[,rep(1, length(mud))]
dmax2 <- 2

all_chla <- z2_rep_mat/dmax2 + mud_rep_mat*(1-z2_rep_mat/dmax2-exp((-2*e*z2_rep_mat)/dmax2))

rm(z2_rep_mat,mud_rep_mat,z2_mat,mud_mat)

#CALCULATE RELATIVE CHL-A CONCENTRATION PER LAYER (SO NON-CUMULATIVE)
all_relative_chla <- all_chla[2:nrow(all_chla),] - all_chla[1:(nrow(all_chla)-1),]


## relative to absolute chla distribution
all_absolute_chla<-  data.frame(matrix(ncol = 0, nrow = 200))
#for (zz in 1: length(all_relative_chla)){
#for (zz in 1: 10){
#absolute_chla <- all_relative_chla[,zz] * Chla[zz] / sum(all_relative_chla[,zz])

# all_absolute_chla <- cbind(all_absolute_chla,absolute_chla)}
Chla_mat <- t(as.matrix(Chla))
Chla_rep_mat <- Chla_mat[rep(1, 200),]
rel_chla_sum <- colSums(all_relative_chla)
rel_chla_sum_mat<- t(as.matrix(rel_chla_sum))
sum_rep_mat <- rel_chla_sum_mat[rep(1, 200),]

Chla_rep <- Chla_mat[rep(1, 200),]

all_absolute_chla <- all_relative_chla*Chla_rep/sum_rep_mat

rm(Chla_mat,Chla_rep_mat,rel_chla_sum,rel_chla_sum_mat,sum_rep_mat,Chla_rep,all_relative_chla,all_chla)
########CALCULATE PRODUCTION USING EP FORMULATION----------------------------------------------------------------------

#Calculate production

P_pixels <- matrix(nrow=0,ncol=1)

for (kk in 1:ncol(emersion_WES)){  
  
  Ez <- light_rep_mat*exp(-Kd_station[kk]*z) 
  
  chla <- as.matrix(all_absolute_chla[,kk])
  chla_rep_mat <- chla[,rep(1, 720)]
  
  response <- Ez/((1/(alpha*eopt^2))*Ez^2+t((1/ps-2/(alpha*eopt))*t(Ez))+(1/alpha))*chla_rep_mat
  response_sum <- (colSums(response)-RES) *0.043
  PP_allpixels_time <- emersion_WES[,kk]*response_sum
  P_pixels <- rbind(P_pixels,sum(PP_allpixels_time))
  print(kk)
}

Production_allpixels_WES <- cbind(P_pixels,LS8_band5_WES$RD_X,LS8_band5_WES$RD_Y)

Production_allpixels_WES<-as.data.frame(Production_allpixels_WES)

colnames(Production_allpixels_WES)[1] <- "Production_1203_1004_2015"
colnames(Production_allpixels_WES)[2] <- "RD_X"
colnames(Production_allpixels_WES)[3] <- "RD_Y"

#Calculate production per hour


Production_allpixels_WES$production_hour <- Production_allpixels_WES$Production_1203_1004_2015/720

production_WES_hour <- Production_allpixels_WES[,-(1)]

production_WES_hour$production_hour <- as.numeric(production_WES_hour$production_hour)

production_WES_hour_nona <- production_WES_hour[!is.na(production_WES_hour$production_hour),]

write.table(production_WES_hour_nona, "//working/directory/Production_2D_WES_.txt", sep=",")

hist(Production_allpixels_WES$Production_1203_1004_2015)
