###Tide estimation model : USES GAUGE DATA WITH AVERAGE WATER HEIGHTS PER HOUR

#Install and load required R packages ###################################################################################################
require(plyr)

#Set working directory ##################################################################################################################
#Insert your local working directory containing the provided input data files
setwd("//working/directory")

#Load input data files ##################################################################################################################

###Load coordinates (coordinate system: RD new) of tide gauges
#Format = .txt (or .csv files) containing at least four columns with the name of the tide gauge, X coordinates, Y coordinates and the name of the estuary
#the names of the tide gauges should be ordered alphabetically 
coord_gauges <- read.csv("//working/directory/coord_waterbase_stations.txt" ,sep=",",colClasses=c("character","numeric","numeric","character"))

#Load data files containing water heights at available tide gauges, preferably containing a measurement value every 10 minutes
#Format = .txt (or .csv files) containing at least four columns with the name of the tide gauge, time, water height and the unit
#Names of tide gauges must be ordered alphabetically
#Tide gauge data for the Netherlands can be downloaded here: https://waterinfo.rws.nl/#!/nav/index/
tides_WES <- read.csv("//working/directory/all_gauges_WES.csv")

###Load coordinates of pixels of which we want to know the water heights
coord_pixels_WES <- read.csv("//working/directory/l1_WES_noveg.txt",colClasses=c(rep("numeric",6)))

#Convert data to correct format #########################################################################################################

#Westerschelde ##########################################################################################################

#convert the 'time'variable in the water height dataset to a numeric value
tides_WES$tijd <- as.character(tides_WES$tijd)

#Create new column containing a variable with tide gauge + date and time information combined
tides_WES$get_average_per_hour <- paste(substr(tides_WES$locatie,1,4),tides_WES$datum,substr(tides_WES$tijd,1,2),sep="_")

#calculate the mean water height per hour from the 10 minute water height time series data
#The function takes the average of each 'hour number', so the average water height at 2:00 PM is the average of all water height values from 2:00-3:00
WES_water_heights_hour <- ddply(tides_WES,~get_average_per_hour,summarise,mean=mean(waarde),sd=sd(waarde))

#Make a column with tide gaugs nr's instead of names
Gauge_nrs <- matrix(ncol = 1, nrow = length(WES_water_heights_hour$get_average_per_hour))

for (d in 1:length(WES_water_heights_hour$get_average_per_hour)){
  if (substr(WES_water_heights_hour$get_average_per_hour[d],1,4) == "Bath"){
    gauge_nr <- 1
  }
  
  if (substr(WES_water_heights_hour$get_average_per_hour[d],1,4) == "Cadz"){
    gauge_nr <- 2
  }
  
  if (substr(WES_water_heights_hour$get_average_per_hour[d],1,4) == "Hans"){
    gauge_nr <- 3
  }
  
  if (substr(WES_water_heights_hour$get_average_per_hour[d],1,4) == "Tern"){
    gauge_nr <- 4
  }  
  
  if (substr(WES_water_heights_hour$get_average_per_hour[d],1,4) == "Vlis"){
    gauge_nr <- 5
  }  
  
  if (substr(WES_water_heights_hour$get_average_per_hour[d],1,4) == "West"){
    gauge_nr <- 6
  }  
  
  Gauge_nrs[d] <- gauge_nr 
}
#Add column with gauge nr information to the data frame containing the water height data per hour
WES_water_heights_hour$Gauge_nr <- Gauge_nrs

#Add the estuary name to the pixel coordinate dataset (in case of multiple estuaries)
coord_pixels_WES$Basin <- rep("WES",length(coord_pixels_WES$RD_X))


###For each pixel, get distance between tide gauge and pixel (using Pythagorean theorem) ########################################################

getdistance <- function(gaugepointX,gaugepointY,PointX, PointY){   
  deltaX <- (gaugepointX - PointX) * (gaugepointX - PointX)
  deltaY <- (gaugepointY - PointY) * (gaugepointY - PointY)
  deltaZ <- sqrt(abs(deltaX) + abs(deltaY))
  return (deltaZ)}

###Westerschelde ###################################################################################################

#Select data from Westerschelde estuary
coord_WES <- subset(coord_gauges,Basin == "WES")

#Get distance of each pixel to the three nearest tide gauges
distances_WES <- data.frame(matrix(ncol = 3, nrow = length(coord_pixels_WES$RD_X)))
i_gauges_WES <- matrix(ncol = 3, nrow = length(coord_pixels_WES$RD_X))

for (d in 1:length(coord_pixels_WES$RD_X))  {
  distances <- matrix(ncol = 0, nrow = 0)
  for (e in 1:length(coord_WES$X)){
    
    dist <- getdistance(coord_WES$X[e],coord_WES$Y[e],coord_pixels_WES$RD_X[d],coord_pixels_WES$RD_Y[d])
    distances[e]<- dist}
    sorted_dist <- sort.int(distances,index.return=TRUE)
    distances_WES[d,] <- sorted_dist$x[1:3]
    i_gauges_WES[d,] <- sorted_dist$ix[1:3]
}

colnames(distances_WES)[1] <-"Distance to nearest gauge (m)"
colnames(distances_WES)[2] <-"Distance to second nearest gauge (m)"
colnames(distances_WES)[3] <-"Distance to third nearest gauge (m)"


### Calculate water height using inverse distance weighting ##############################################################################
#Calculate a time series of water heights for each pixel

runidw <- function(data){
  values <- data[1:3]
  dists <- data[4:6]
  weights <- 1/dists
  val <- sum(values * weights)/sum(weights)
  return (val)
}

select_dataset <- function(index_gauge,water_heights,water_heights_gauge_nr){
  gauge_subset <- water_heights[index_gauge == water_heights_gauge_nr]  
  return(gauge_subset)
}

#Westerschelde ###########################################################################################################################

#Get water height time series of the three nearest gauges for each pixel
water_heights_1_WES <- sapply(i_gauges_WES[,1],select_dataset,WES_water_heights_hour$mean,WES_water_heights_hour$Gauge_nr)
water_heights_2_WES <- sapply(i_gauges_WES[,2],select_dataset,WES_water_heights_hour$mean,WES_water_heights_hour$Gauge_nr)
water_heights_3_WES <- sapply(i_gauges_WES[,3],select_dataset,WES_water_heights_hour$mean,WES_water_heights_hour$Gauge_nr)

distances_WES <- as.matrix(distances_WES)

#repeat each distance by the number of water heights (length of time series) available for each tide gauge
distances_WES_time <- distances_WES[rep(1:nrow(distances_WES), times = nrow(water_heights_1_WES)), ]

#put water heights in the correct format: one column for each water height
water_heights_1_vec_WES <- c(water_heights_1_WES[,1:length(water_heights_1_WES[1,])]) 
water_heights_2_vec_WES <- c(water_heights_2_WES[,1:length(water_heights_2_WES[1,])]) 
water_heights_3_vec_WES <- c(water_heights_3_WES[,1:length(water_heights_3_WES[1,])]) 

rm(water_heights_1_WES,water_heights_2_WES,water_heights_3_WES)

#merge the water height of all three tide gauges to one data frame
water_heights_WES <- cbind(water_heights_1_vec_WES,water_heights_2_vec_WES,water_heights_3_vec_WES)
rm(water_heights_1_vec_WES,water_heights_2_vec_WES,water_heights_3_vec_WES)

#merge water height and distance information to one data frame
heights_dists_WES_hour <- cbind(water_heights_WES,distances_WES_time)

rm(water_heights_WES,distances_WES_time)
rm(distances_WES,i_gauges_WES)
rm(coord_pixels,coord_pixels_WES,tides_WES)

#split dataset if necessary (to prevent PC memory issues during the inverse distance weight calculation)
heights_dists_WES_hour1 <- heights_dists_WES_hour[1:9672000,] 
heights_dists_WES_hour2 <-heights_dists_WES_hour[9672001:19344002,]
heights_dists_WES_hour3 <-heights_dists_WES_hour[19344003:29016003,]
heights_dists_WES_hour4 <-heights_dists_WES_hour[29016004:38688004,]
heights_dists_WES_hour5 <-heights_dists_WES_hour[38688004:48360005]
heights_dists_WES_hour6 <-heights_dists_WES_hour[48360005:59592168,]

rm(heights_dists_WES_hour)

#Inverse distance weighting
heights_pixels_time_WES_hour1 <- apply(heights_dists_WES_hour1,1,runidw)
rm(heights_dists_WES_hour1)
heights_pixels_time_WES_hour2 <- apply(heights_dists_WES_hour2,1,runidw) 
rm(heights_dists_WES_hour2)
heights_pixels_time_WES_hour3 <- apply(heights_dists_WES_hour3,1,runidw) 
rm(heights_dists_WES_hour3)
heights_pixels_time_WES_hour4 <- apply(heights_dists_WES_hour4,1,runidw) 
rm(heights_dists_WES_hour4)
heights_pixels_time_WES_hour5 <- apply(heights_dists_WES_hour5,1,runidw) 
rm(heights_dists_WES_hour5)
heights_pixels_time_WES_hour6 <- apply(heights_dists_WES_hour6,1,runidw) 
rm(heights_dists_WES_hour6)


################################################################################################################################ 

#Create matrix with one column per pixel (744 = length of the time series)

#Westerschelde ####################################################################################

rm(heights_pixels_time_WES_hour)

heights_pixels_WES_hour1_mat <- matrix(heights_pixels_time_WES_hour1,nrow=744,ncol=length(heights_pixels_time_WES_hour1)/744)
heights_pixels_WES_hour2_mat <- matrix(heights_pixels_time_WES_hour2,nrow=744,ncol=length(heights_pixels_time_WES_hour2)/744)
heights_pixels_WES_hour3_mat <- matrix(heights_pixels_time_WES_hour3,nrow=744,ncol=length(heights_pixels_time_WES_hour3)/744)
heights_pixels_WES_hour4_mat <- matrix(heights_pixels_time_WES_hour4,nrow=744,ncol=length(heights_pixels_time_WES_hour4)/744)
heights_pixels_WES_hour5_mat <- matrix(heights_pixels_time_WES_hour5,nrow=744,ncol=length(heights_pixels_time_WES_hour5)/744)
heights_pixels_WES_hour6_mat <- matrix(heights_pixels_time_WES_hour6,nrow=744,ncol=length(heights_pixels_time_WES_hour6)/744)



heights_pixels_WES_hour_mat <- cbind(heights_pixels_WES_hour1_mat,heights_pixels_WES_hour2_mat,heights_pixels_WES_hour3_mat,heights_pixels_WES_hour4_mat,heights_pixels_WES_hour5_mat,heights_pixels_WES_hour6_mat)

rm(heights_pixels_time_WES_hour1,heights_pixels_time_WES_hour2,heights_pixels_time_WES_hour3,heights_pixels_time_WES_hour4,heights_pixels_time_WES_hour5)
rm(heights_pixels_WES_hour1_mat,heights_pixels_WES_hour2_mat,heights_pixels_WES_hour3_mat,heights_pixels_WES_hour4_mat,heights_pixels_WES_hour5_mat,heights_pixels_WES_hour6_mat,heights_pixels_WES_hour7_mat)
### Get emersion time using bathymetry map ###################################################################################

#Load required paxkages and bathymetry map
require(rgdal)
require(raster)

bathy_WES <- raster("//working/directory/Hoogtekaarten/wschelde_20151.img")

pixelsWES <- as.data.frame(cbind(coord_pixels_WES$RD_X,coord_pixels_WES$RD_Y))

bathy_pixels_WES <- extract(bathy_WES,pixelsWES)

bathy_pixels_WES<- bathy_pixels_WES*100



#Get emersion (1) vs. no emersion (0) Western Scheldt



emersion_WES <- matrix(ncol = length(bathy_pixels_WES), nrow = length(heights_pixels_WES_hour_mat[,1]))

for (ii in 1:length(bathy_pixels_WES)){
  for (jj in 1:length(heights_pixels_WES_hour_mat[,1])){
    
    if  (heights_pixels_WES_hour_mat[jj,ii] > bathy_pixels_WES[ii]){
      
      e_point <- 0
    }
    
    else if (heights_pixels_WES_hour_mat[jj,ii] < bathy_pixels_WES[ii]){
      
      e_point <- 1
    }
    emersion_WES[jj,ii] <- e_point
  }  
  print(ii)
}

hist(colSums(emersion_WES)/744)

save(emersion_WES,file="emersion_WES_hour_firsthour_NOTexcl_noveg.RData")


#change first 1 after a 0 in to a 0

emersion_WES[emersion_WES == 0] <- NA

for (ii in 26750:ncol(emersion_WES)){
  for (jj in 2:nrow(emersion_WES)){
    
    
      
      if  (!is.na(emersion_WES[jj,ii]) &&  is.na(emersion_WES[jj-1,ii])){
        
        emersion_WES[jj,ii] <- 0
      }
      
      
     }
  print(ii)
}
       
emersion_WES[is.na(emersion_WES)] <- 0

emersion_hour <- as.data.frame(cbind(as.character(tides_OOS[[1]]$datum),as.character(tides_OOS[[1]]$tijd),emersion))
colnames(emersion_hour) <- c("DateTime",coord_pixels$Loc)

save(emersion_WES,file="emersion_WES_hour_firsthour_excl.RData")


colnames(emersion_landsat)[3:56] <- coord_pixels_WES$Loc
emersion_landsat <- emersion_hour[208:220,]

save(emersion_landsat,file="emersion_plots032015.RData")











